/***************************************************************************
                         sWM.c  -  description
                         ---------------------
    begin                : Wed May 22 10:12:15 CEST 2001
    copyright            : (C) 2000-2002 by Robert Sperling
    email                : sperling@small-window-manager.de
    version              : v1.3.5
 ***************************************************************************/
/*!
\file sWM.c
\author Robert Sperling <sperling@small-window-manager.de>
\version 1.3.5
\brief main part of sWM and documentation

\verbatim
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
\endverbatim
*/

#include "sWM.h"
#include "errorHandler.c"

#include "gnome.c"
#include "tools.c"
#include "eventHandler.c"
#include "client.c"
#include "init.c"
//#include "load.c"

/***************************************************************************
 *   Main - method                                                         *
 ***************************************************************************/

//  main
//! entrypoint and command line argument check
/*!
\param argc number of command line arguments
\param array of command line arguments 
*/
int main(int argc, char *argv[])
{ 
#ifndef MINIMAL
  int counter;
  
  for (counter = 1; counter < argc; counter++) 
  { if (strcmp(argv[counter], "-display")==0 && counter+1<argc)
    { displayOption = argv[++counter]; 
    }
    else if (strcmp(argv[counter], "-version")==0)
    {
#ifdef STD_IO    
      fprintf(stdout,"sWM version: %s, (c) 2000-2002 by Robert Sperling\n",SWMVERSION); 
      fprintf(stdout,"Distributed under the terms of the GPL.\n");
#endif
      exit(0);
    }
    else 
    {     
#ifdef STD_IO    
      fprintf(stderr,"usage: sWM [-display display]\n"); 
#endif
      exit(2);
    }// end else 
  }// end for
#endif // NOT MINIMAL
 
  initSWM();

#ifdef GNOME  
  initGnomeCompliance();
#endif  

  scanCurrentWindows();
  //XSynchronize(display,false);
  sWMEventLoop();   

  sWMQuitNicely();
  exit(0); 
}// end main

// ***************************************************************************

//  sWMQuitNicely
//! cleanup and exit
void sWMQuitNicely()
{ Client *client;
  unsigned int count, maxCount;
  Window w1,w2, *windowList=NULL;

#ifdef TRACE
  trace (1,"sWMQuitNicely");
#endif
#ifdef PIXMAPS
  releasePixmaps();
#endif

  XQueryTree (display, rootWindow, &w1, &w2, &windowList, &maxCount);

  for (count = 0; count < maxCount; count++)
  { client = getClient(windowList[count], anyWindow);
    if (client) 
    { removeParent(client, true); // map windows again
    }
  }
  if (windowList) XFree (windowList);
  
#ifdef FONTS  
  XFreeFont(display,sWMFont);
#endif  
#ifdef DEBUG
  err("cleanup succesful, terminating sWM");
#endif  
  exit(0);
}// end sWMQuitNicely 

// ***************************************************************************
// Additional file descriptions and documentation
// ***************************************************************************

/*!
\file sWM.depend
\author Robert Sperling <sperling@small-window-manager.de>
\brief internal dependency definitions

\verbatim
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
\endverbatim 
*/

/*!
\file config.h
\author Robert Sperling <sperling@small-window-manager.de>
\brief main configuration file for sWM

\verbatim
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
\endverbatim 
*/

/*!
\file keybindings.conf
\author Robert Sperling <sperling@small-window-manager.de>
\version 1.3.5
\brief keyboard mappings if sWM

\verbatim
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
\endverbatim 
*/
/*! \page ChangeLog ChangeLog
sWM Changelog for version 1.3.0-pre* and above
\include ./changelog
*/

/*!\page README README
\include ./README
*/

/*!\page AUTHORS AUTHORS
\include ./AUTHORS
*/

/*!\page Features sWM-Features
\include ./FEATURES
*/

/*!\page sWM-FAQ FAQ
\include ./SWM-FAQ
*/

/*!\page X-speedup-FAQ X11-speedup-FAQ
\include ./X-SPEEDUP-FAQ
*/

/*!\page License License
\include ./COPYING
*/


/*! \mainpage Small Window Manager
\author (c) 2000-2002 by Robert Sperling <sperling@small-window-manager.de>
\author <http://www.small-window-manager.de>

\section sWMIntroduction Introduction
sWM is a small window manager for X11. It was designed for laptops and other
"handicapped" computers with litte amount of memory and small screen sizes.
But sWM runs perfectly on all other computer systems providing rather unusual
features eating up less memory than many other window managers. 

\section sWMLicense License
\verbatim
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
\endverbatim 

\section Installation Installation
There are three ways to get sWM binaries:

1. Run the configure script, answer the questions

2. edit config.h by hand, you can find some hints in the configure script,
also edit config.mk

3. go with the defaults and get the most important features
(choose in configure script) 


Finally type "make" and "make install" to install.

If something goes wrong, mail me. :-)  

*/

